@@ -13,10 +13,11 @@ at the top-level directory.
  * \brief Driver program for PZGSSVX example
  *
  * <pre>
- * -- Distributed SuperLU routine (version 6.1) --
+ * -- Distributed SuperLU routine (version 8.1.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * November 1, 2007
  * December 6, 2018
+ * AUgust 27, 2022  Add batch option
  * </pre>
  */
 
@@ -58,12 +59,21 @@ int main(int argc, char *argv[])
     double   *berr;
     doublecomplex   *b, *xtrue;
     int    m, n;
-    int      nprow, npcol, lookahead, colperm, rowperm, ir;
+    int      nprow, npcol, lookahead, colperm, rowperm, ir, symbfact, batch;
     int      iam, info, ldb, ldx, nrhs;
     char     **cpp, c, *postfix;;
     FILE *fp, *fopen();
     int cpp_defs();
     int ii, omp_mpi_level;
+    int ldumap, myrank, p; /* The following variables are used for batch solves */
+    int*    usermap;
+    float result_min[2];
+    result_min[0]=1e10;
+    result_min[1]=1e10;
+    float result_max[2];
+    result_max[0]=0.0;
+    result_max[1]=0.0;
+    MPI_Comm SubComm;
 
     nprow = 1;  /* Default process rows.      */
     npcol = 1;  /* Default process columns.   */
@@ -72,6 +82,8 @@ int main(int argc, char *argv[])
     colperm = -1;
     rowperm = -1;
     ir = -1;
+    symbfact = -1;
+    batch = 0;
 
     /* ------------------------------------------------------------
        INITIALIZE MPI ENVIRONMENT. 
@@ -110,8 +122,12 @@ int main(int argc, char *argv[])
                         break;
               case 'q': colperm = atoi(*cpp);
                         break;
+	      case 's': symbfact = atoi(*cpp);
+		        break;
               case 'i': ir = atoi(*cpp);
                         break;
+              case 'b': batch = atoi(*cpp);
+                        break;
 	    }
 	} else { /* Last arg is considered a filename */
 	    if ( !(fp = fopen(*cpp, "r")) ) {
@@ -121,32 +137,87 @@ int main(int argc, char *argv[])
 	}
     }
 
-    /* ------------------------------------------------------------
-       INITIALIZE THE SUPERLU PROCESS GRID. 
-       ------------------------------------------------------------*/
-    superlu_gridinit(MPI_COMM_WORLD, nprow, npcol, &grid);
+    if ( batch ) { /* in the batch mode: create multiple SuperLU grids,
+		      each grid solving one linear system. */
+	/* ------------------------------------------------------------
+	   INITIALIZE MULTIPLE SUPERLU PROCESS GRIDS. 
+	   ------------------------------------------------------------*/
+        MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
+        usermap = SUPERLU_MALLOC(nprow*npcol * sizeof(int));
+        ldumap = nprow;
+        int color = myrank/(nprow*npcol); /* Assuming each grid uses the same number of nprow and npcol */
+	MPI_Comm_split(MPI_COMM_WORLD, color, myrank, &SubComm);
+        p = 0;    
+        for (int i = 0; i < nprow; ++i)
+    	    for (int j = 0; j < npcol; ++j) usermap[i+j*ldumap] = p++;
+        superlu_gridmap(SubComm, nprow, npcol, usermap, ldumap, &grid);
+        SUPERLU_FREE(usermap);
+
+#ifdef GPU_ACC
+        /* Binding each MPI to a GPU device */
+        char *ttemp;
+        ttemp = getenv ("SUPERLU_BIND_MPI_GPU");
+
+        if (ttemp) {
+	    int devs, rank;
+	    MPI_Comm_rank(MPI_COMM_WORLD, &rank); // MPI_COMM_WORLD needs to be used here instead of SubComm
+	    gpuGetDeviceCount(&devs);  // Returns the number of compute-capable devices
+	    gpuSetDevice(rank % devs); // Set device to be used for GPU executions
+        }
+
+        // This is to initialize GPU, which can be costly. 
+        double t1 = SuperLU_timer_();                       
+        gpuFree(0);
+        double t2 = SuperLU_timer_();    
+        if(!myrank)printf("first gpufree time: %7.4f\n",t2-t1);
+        gpublasHandle_t hb;           
+        gpublasCreate(&hb);
+        if(!myrank)printf("first blas create time: %7.4f\n",SuperLU_timer_()-t2);
+        gpublasDestroy(hb);
+#endif
+        // printf("grid.iam %5d, myrank %5d\n",grid.iam,myrank);
+        // fflush(stdout);
+
+    } else {
+        /* ------------------------------------------------------------
+           INITIALIZE THE SUPERLU PROCESS GRID.
+           ------------------------------------------------------------ */
+        superlu_gridinit(MPI_COMM_WORLD, nprow, npcol, &grid);
 	
+#ifdef GPU_ACC
+        MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
+        double t1 = SuperLU_timer_();                       
+        gpuFree(0);
+        double t2 = SuperLU_timer_();    
+        if(!myrank)printf("first gpufree time: %7.4f\n",t2-t1);
+        gpublasHandle_t hb;           
+        gpublasCreate(&hb);
+        if(!myrank)printf("first blas create time: %7.4f\n",SuperLU_timer_()-t2);
+        gpublasDestroy(hb);
+#endif
+    }
+    
     if(grid.iam==0){
 	MPI_Query_thread(&omp_mpi_level);
-    switch (omp_mpi_level) {
-      case MPI_THREAD_SINGLE:
+        switch (omp_mpi_level) {
+          case MPI_THREAD_SINGLE:
 		printf("MPI_Query_thread with MPI_THREAD_SINGLE\n");
 		fflush(stdout);
-	break;
-      case MPI_THREAD_FUNNELED:
+	        break;
+          case MPI_THREAD_FUNNELED:
 		printf("MPI_Query_thread with MPI_THREAD_FUNNELED\n");
 		fflush(stdout);
-	break;
-      case MPI_THREAD_SERIALIZED:
+	        break;
+          case MPI_THREAD_SERIALIZED:
 		printf("MPI_Query_thread with MPI_THREAD_SERIALIZED\n");
 		fflush(stdout);
-	break;
-      case MPI_THREAD_MULTIPLE:
+	        break;
+          case MPI_THREAD_MULTIPLE:
 		printf("MPI_Query_thread with MPI_THREAD_MULTIPLE\n");
 		fflush(stdout);
-	break;
+	        break;
+        }
     }
-	}
 	
     /* Bail out if I do not belong in the grid. */
     iam = grid.iam;
@@ -220,6 +291,7 @@ int main(int argc, char *argv[])
     if (colperm != -1) options.ColPerm = colperm;
     if (lookahead != -1) options.num_lookaheads = lookahead;
     if (ir != -1) options.IterRefine = ir;
+    if (symbfact != -1) options.ParSymbFact = symbfact;
 
     if (!iam) {
 	print_options_dist(&options);
@@ -257,7 +329,6 @@ int main(int argc, char *argv[])
        DEALLOCATE STORAGE.
        ------------------------------------------------------------*/
 
-    PStatFree(&stat);
     Destroy_CompRowLoc_Matrix_dist(&A);
     zScalePermstructFree(&ScalePermstruct);
     zDestroy_LU(n, &grid, &LUstruct);
@@ -272,8 +343,25 @@ int main(int argc, char *argv[])
        RELEASE THE SUPERLU PROCESS GRID.
        ------------------------------------------------------------*/
 out:
+    if ( batch ) {
+        result_min[0] = stat.utime[FACT];   
+        result_min[1] = stat.utime[SOLVE];  
+        result_max[0] = stat.utime[FACT];   
+        result_max[1] = stat.utime[SOLVE];    
+        MPI_Allreduce(MPI_IN_PLACE, result_min, 2, MPI_FLOAT,MPI_MIN, MPI_COMM_WORLD);
+        MPI_Allreduce(MPI_IN_PLACE, result_max, 2, MPI_FLOAT,MPI_MAX, MPI_COMM_WORLD);
+        if (!myrank) {
+            printf("returning data:\n");
+            printf("    Factor time over all grids.  Min: %8.4f Max: %8.4f\n",result_min[0], result_max[0]);
+                        printf("    Solve time over all grids.  Min: %8.4f Max: %8.4f\n",result_min[1], result_max[1]);
+            printf("**************************************************\n");
+            fflush(stdout);
+        }
+    }
+    
     superlu_gridexit(&grid);
-
+    if ( iam != -1 ) PStatFree(&stat);
+    
     /* ------------------------------------------------------------
        TERMINATES THE MPI EXECUTION ENVIRONMENT.
        ------------------------------------------------------------*/
