@@ -81,9 +81,9 @@ at the top-level directory.
  * Versions 4.x and earlier do not include a #define'd version numbers.
  */
 #define SUPERLU_DIST_MAJOR_VERSION     8
-#define SUPERLU_DIST_MINOR_VERSION     0
-#define SUPERLU_DIST_PATCH_VERSION     0
-#define SUPERLU_DIST_RELEASE_DATE      "May 22, 2022"
+#define SUPERLU_DIST_MINOR_VERSION     1
+#define SUPERLU_DIST_PATCH_VERSION     1
+#define SUPERLU_DIST_RELEASE_DATE      "October 1, 2022"
 
 #include "superlu_dist_config.h"
 
@@ -1064,6 +1064,7 @@ extern void   superlu_gridmap(MPI_Comm, int, int, int [], int, gridinfo_t *);
 extern void   superlu_gridexit(gridinfo_t *);
 extern void   superlu_gridinit3d(MPI_Comm Bcomm,  int nprow, int npcol, int npdep,
 				 gridinfo3d_t *grid) ;
+extern void   superlu_gridmap3d(MPI_Comm, int, int, int, int [], gridinfo3d_t *);
 extern void   superlu_gridexit3d(gridinfo3d_t *grid);
 
 extern void   set_default_options_dist(superlu_dist_options_t *);
